{-# LANGUAGE DataKinds          #-}
{-# LANGUAGE DeriveDataTypeable #-}
{-# LANGUAGE DeriveGeneric      #-}
{-# LANGUAGE FlexibleInstances  #-}
{-# LANGUAGE NoImplicitPrelude  #-}
{-# LANGUAGE OverloadedStrings  #-}
{-# LANGUAGE RecordWildCards    #-}
{-# LANGUAGE TypeFamilies       #-}
{-# LANGUAGE TypeOperators      #-}

{-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
{-# OPTIONS_GHC -fno-warn-unused-binds      #-}
{-# OPTIONS_GHC -fno-warn-unused-imports    #-}

-- |
-- Module      : Network.Google.Resource.Gmail.Users.Settings.GetLanguage
-- Copyright   : (c) 2015-2016 Brendan Hay
-- License     : Mozilla Public License, v. 2.0.
-- Maintainer  : Brendan Hay <brendan.g.hay@gmail.com>
-- Stability   : auto-generated
-- Portability : non-portable (GHC extensions)
--
-- Gets language settings.
--
-- /See:/ <https://developers.google.com/gmail/api/ Gmail API Reference> for @gmail.users.settings.getLanguage@.
module Network.Google.Resource.Gmail.Users.Settings.GetLanguage
    (
    -- * REST Resource
      UsersSettingsGetLanguageResource

    -- * Creating a Request
    , usersSettingsGetLanguage
    , UsersSettingsGetLanguage

    -- * Request Lenses
    , usglUserId
    ) where

import           Network.Google.Gmail.Types
import           Network.Google.Prelude

-- | A resource alias for @gmail.users.settings.getLanguage@ method which the
-- 'UsersSettingsGetLanguage' request conforms to.
type UsersSettingsGetLanguageResource =
     "gmail" :>
       "v1" :>
         "users" :>
           Capture "userId" Text :>
             "settings" :>
               "language" :>
                 QueryParam "alt" AltJSON :>
                   Get '[JSON] LanguageSettings

-- | Gets language settings.
--
-- /See:/ 'usersSettingsGetLanguage' smart constructor.
newtype UsersSettingsGetLanguage =
  UsersSettingsGetLanguage'
    { _usglUserId :: Text
    }
  deriving (Eq, Show, Data, Typeable, Generic)


-- | Creates a value of 'UsersSettingsGetLanguage' with the minimum fields required to make a request.
--
-- Use one of the following lenses to modify other fields as desired:
--
-- * 'usglUserId'
usersSettingsGetLanguage
    :: UsersSettingsGetLanguage
usersSettingsGetLanguage = UsersSettingsGetLanguage' {_usglUserId = "me"}


-- | User\'s email address. The special value \"me\" can be used to indicate
-- the authenticated user.
usglUserId :: Lens' UsersSettingsGetLanguage Text
usglUserId
  = lens _usglUserId (\ s a -> s{_usglUserId = a})

instance GoogleRequest UsersSettingsGetLanguage where
        type Rs UsersSettingsGetLanguage = LanguageSettings
        type Scopes UsersSettingsGetLanguage =
             '["https://mail.google.com/",
               "https://www.googleapis.com/auth/gmail.modify",
               "https://www.googleapis.com/auth/gmail.readonly",
               "https://www.googleapis.com/auth/gmail.settings.basic"]
        requestClient UsersSettingsGetLanguage'{..}
          = go _usglUserId (Just AltJSON) gmailService
          where go
                  = buildClient
                      (Proxy :: Proxy UsersSettingsGetLanguageResource)
                      mempty
